package com.steria.digitalsignature.config;


/**
 * @author cvila
 * Cont constants de configuraci que permeten referenciar fcilment els parmetres complexes noms estndard DSS o profiles.
 */
public interface ConstantsApplicacio
{

    // tipus 
    public static final String TYPE_X509Certificate="X509Certificate"; 
    public static final String TYPE_Base64Signature="Base64Signature"; 
    public static final String TYPE_Base64Data="Base64Data"; 
    public static final String TYPE_InlineXML="InlineXML"; 
    public static final String TYPE_DocumentHash="DocumentHash";     
    public static enum TYPE { TYPE_X509Certificate,TYPE_Base64Signature,TYPE_Base64Data,TYPE_InlineXML,TYPE_DocumentHash }
        
    // Atributs de Certificat
    public static final String CERT_NIF="urn:catcert:psis:certificateAttributes:KeyOwnerNIF"; 
    public static final String CERT_Versio="urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:Version"; 
    public static final String CERT_UIDCertificat="urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:SerialNumber"; 
    //    urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:SubjectDistinguishedName:serialNumber
    public static final String CERT_signaturaCA="urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:Signature"; 
    public static final String CERT_algoritmeSignaturaCertificat="urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:SignatureAlgorithm"; 
    public static final String CERT_nomEmissor="urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:IssuerDistinguishedName"; 
    public static final String CERT_nomDePilaPersona="Urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:SubjectDistinguishedName:commonName"; 
    public static final String CERT_nomPersona="urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:SubjectDistinguishedName:givenName"; 
    public static final String CERT_cognomPersona="Urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:SubjectDistinguishedName:surname"; 
    public static final String CERT_algoritmeClauPublica="urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:SubjectPublicKeyAlgorithm"; 
    public static final String CERT_clauPublica="urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:SubjectPublicKey"; 
    public static final String CERT_organitzacio="urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:SubjectDistinguishedName:organizationName"; 
    public static final String CERT_dataIniciCertificat="urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:NotBefore"; 
    public static final String CERT_dataFiCertificat="urn:oasis:names:tc:dss:1.0:profiles:XSS:certificateAttributes:NotAfter"; 
        
    // Atributs de Signatura
    public static final String SIGN_funcioResumAlgoritme="urn:oasis:names:tc:dss:1.0:profiles:XSS:signatureAttributes:DigestAlgorithm"; 
    public static final String SIGN_algoritmeEncriptacioResum="urn:oasis:names:tc:dss:1.0:profiles:XSS:signatureAttributes:DigestEncryptionAlgorithm"; 
    public static final String SIGN_algoritmeSignatura="urn:oasis:names:tc:dss:1.0:profiles:XSS:signatureAttributes:SignatureAlgorithm"; 
    public static final String SIGN_signatura="urn:oasis:names:tc:dss:1.0:profiles:XSS:signatureAttributes:SignatureValue"; 

    // Atributs de Pdf
    public static final String PDF_horaSignaturaPdf="ReturnSigningTime"; 
    public static final String PDF_signant="ReturnSignerIdentity"; 
    public static final String PDF_raoSignatura="ReturnSignatureReason"; 
    
    // Atributs de Segell    
    public static final String TIMESTAMP_tipusSignatura="signatureTypeCMS"; 
    
    //Attribut que conte el XML a guardar com dades de segell de temps d'un document. 
    public static final String TIMESTAMP_signatura="dss:SignatureObject"; 
    
    // Atributs de Protocol
    public static final String PROTOCOL_perfilDSS="urn:oasis:names:tc:dss:1.0:profiles:XSS"; 
    public static final String PROTOCOL_perfilDSS_PDF="urn:oasis:names:tc:dss:1.0:profiles:DSS_PDF"; 
    public static final String PROTOCOL_perfilDSS_Timestmaping="urn:oasis:names:tc:dss:1.0:profiles:timestamping"; 

    // Atributs de Esquema
    public static final String SCHEMA_XMLtimestamp="oasis:names:tc:dss:1.0:core:schema:XMLTimeStampToken"; 
    
    // Operacions
    public static final String OP_politicaSignatura="xss:SignaturePolicy"; 
    public static final String OP_validacioAData="dss:VerificationTime"; 
    public static final String OP_detallsProces="dss:ReturnProcessingDetails"; 
    public static final String OP_horaSignatura="dss:ReturnSigningTime"; 
    public static final String OP_mostrarAtributsSignatura="xss:ReturnSignatureInfo"; 
    public static final String OP_mostrarAtributsCertificat="xss:ReturnX509CertificateInfo"; 
    public static final String OP_informacioAddicional="dss:AdditionalKeyInfo"; 
    public static final String OP_certificatQualificat="xss:RequireQualifiedCertificate"; 
    public static final String OP_politica="dss:ServicePolicy"; 
    public static final String OP_opcionsValidacioCertificat="xss:X509CertificateValidationOptions"; 
    public static final String OP_signarResposta="xss:ReturnSignedResponse"; 
    public static final String OP_transformarDocument="dss:ReturnTransformedDocument"; 
    public static final String OP_completarADES="dss:ReturnUpdatedSignature"; 
    public static final String OP_schemas="dss:Schemas"; 

    
    // Enums que indiquen quins atributs, operacions, etc pot configurar-se en cada cas, en cada TYPE (cert, sign, etc.)
    public static enum X509CertificateAtributs { CERT_NIF,CERT_Versio,CERT_UIDCertificat,CERT_signaturaCA,CERT_algoritmeSignaturaCertificat,
        CERT_nomEmissor, CERT_nomDePilaPersona, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio, CERT_dataIniciCertificat,  CERT_dataFiCertificat, PROTOCOL_perfilDSS }
    public static enum X509CertificateAtributsDefecte { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
    public static enum X509CertificateOperacions { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
    public static enum X509CertificateOperacionsDefecte { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
        
    public static enum Base64SignatureAtributs { CERT_NIF,CERT_Versio,CERT_UIDCertificat,CERT_signaturaCA,CERT_algoritmeSignaturaCertificat,
        CERT_nomEmissor, CERT_nomDePilaPersona, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio, CERT_dataIniciCertificat,  CERT_dataFiCertificat, PROTOCOL_perfilDSS }
    public static enum Base64SignatureAtributsDefecte { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
    public static enum Base64SignatureOperacionsDefecte { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
    public static enum Base64SignatureOperacions { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
            
    public static enum Base64DataAtributs { CERT_NIF,CERT_Versio,CERT_UIDCertificat,CERT_signaturaCA,CERT_algoritmeSignaturaCertificat,
        CERT_nomEmissor, CERT_nomDePilaPersona, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio, CERT_dataIniciCertificat,  CERT_dataFiCertificat, PROTOCOL_perfilDSS }
    public static enum Base64DataAtributsDefecte { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
    public static enum Base64DataOperacionsDefecte { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
    public static enum Base64DataOperacions { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
        
    public static enum InlineXMLAtributs { CERT_NIF,CERT_Versio,CERT_UIDCertificat,CERT_signaturaCA,CERT_algoritmeSignaturaCertificat,
        CERT_nomEmissor, CERT_nomDePilaPersona, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio, CERT_dataIniciCertificat,  CERT_dataFiCertificat, PROTOCOL_perfilDSS }
    public static enum InlineXMLAtributsDefecte { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
    public static enum InlineXMLOperacions { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
    public static enum InlineXMLOperacionsDefecte { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
        
    public static enum DocumentHashAtributs { CERT_NIF,CERT_Versio,CERT_UIDCertificat,CERT_signaturaCA,CERT_algoritmeSignaturaCertificat,
        CERT_nomEmissor, CERT_nomDePilaPersona, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio, CERT_dataIniciCertificat,  CERT_dataFiCertificat, PROTOCOL_perfilDSS }
    public static enum DocumentHashAtributsDefecte { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
    public static enum DocumentHashOperacions{ CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};
    public static enum DocumentHashOperacionsDefecte { CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
        CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
        CERT_clauPublica, CERT_organitzacio};

        // Enums que indiquen quins atributs, operacions, etc pot configurar-se en cada cas, en cada TYPE (cert, sign, etc.)
        public static final String[] X509CertificateAtributs = new String[] { 
            CERT_NIF,CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat,
            CERT_nomEmissor, CERT_nomDePilaPersona, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
            CERT_clauPublica, CERT_organitzacio, CERT_dataIniciCertificat,  CERT_dataFiCertificat, PROTOCOL_perfilDSS };
        
        public static final String[] X509CertificateAtributsDefecte = new String[] {
            CERT_NIF, CERT_Versio, CERT_UIDCertificat, CERT_signaturaCA, CERT_algoritmeSignaturaCertificat, 
            CERT_nomEmissor, CERT_nomPersona, CERT_cognomPersona, CERT_algoritmeClauPublica, 
            CERT_clauPublica, CERT_organitzacio};
        
        public static final String[] X509CertificateOperacions = new String[] {
            OP_detallsProces, OP_mostrarAtributsCertificat };
        
        public static final String[] X509CertificateOperacionsDefecte = new String[] {
            OP_detallsProces, OP_mostrarAtributsCertificat };

        public static final String[] SignatureCMSAtributsDefecte = new String[] {
            SIGN_funcioResumAlgoritme, SIGN_algoritmeEncriptacioResum, SIGN_algoritmeSignatura, SIGN_signatura};    

        // keys missatges de resposta
        public static final String MSG_ResultMsg="dss:ResultMessage"; 
        public static final String MSG_ResultMajor="dss:ResultMajor"; 
        public static final String MSG_ResultMinor="dss:ResultMinor"; 
        public static final String MSG_ValidityInterval="ValidityInterval"; 
        public static final String MSG_IssuerTrust="IssuerTrust"; 
        public static final String MSG_RevocationStatus="dss:RevocationStatus"; 
     
       // Objetos en memoria VOs
    public static final String VO_DATOS_INI=  "vo_datos_ini";    
    public static final String VO_FICHA_PORCENTAJES=  "vo_ficha_porcentajes";    
    public static final String VO_FICHA_INCIDENCIAS=  "vo_ficha_incidencias";    
}
