package com.steria.digitalsignature.core;

import java.util.HashMap;
import java.util.Hashtable;

import oasis.names.tc.dss._1_0.core.wsdl.SOAPport;
import oasis.names.tc.dss._1_0.core.wsdl.digitalSignatureServiceClient;

import org.apache.commons.httpclient.contrib.ssl.AuthSSLProtocolSocketFactory;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.protocol.Protocol;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.XmlOptions;

import x0CoreSchema.oasisNamesTcDss1.InternationalStringType;
import x0CoreSchema.oasisNamesTcDss1.ResponseBaseType;

import com.steria.digitalsignature.config.ConstantsApplicacio;
import com.steria.digitalsignature.properties.ConfigProperties;

/**
 * @author cvilamat
 *
 * El core del projecte est organitzat de forma que hi ha una classe Base (pare) de totes, que cont mtodes i atributs aplicables a totes les classes. Daquesta hereten dues classes una de Creaci i un altre de Validaci, aquestes dues a la seva hora hereten, sobreescriuen i afegeixen ms mtodes especfics ja sigui per les classes de creaci de signatura com per les de validaci respectivament.
 * BaseSignatura: s la classe de base que heretaran totes les classes daquest projecte i cont les funcionalitats base que poden requerir qualsevol classe, com retornar el un map amb les respostes de la crida al servei de PSIS, carregar la url de crida al servei de PSIS i inicialitzar la connexi, o carregar el fitxer de propietats de configuraci del mdul i crear els atributs imprescindibles
 */
public class BaseSignatura
{
    private static final Log log = LogFactory.getLog(BaseSignatura.class);
    
    protected ConfigProperties propFile = null;
    
    protected XmlOptions options = null;
    protected String soapURL = ""; 
    protected SOAPport port;
    
    public BaseSignatura() {
        propFile = ConfigProperties.getInstance(true);
        String soapURL = propFile.getProperty(ConfigProperties.PROP_SOAP_URL);
        
        log.info("BaseSignatura (constructor) init. URL = " + soapURL);  
        
        this.options = new XmlOptions();
        
        //Establecer la URL por defecto.
        this.setSoapURL(soapURL);
        
        boolean registerProtocol=true;
        try
        {
                Protocol.getProtocol("psishttps");
                registerProtocol=false;
        }
        catch (Exception e)
        {
        }
        
         if (registerProtocol)       
        {
            PostMethod httppost = null;
            
            // Registrar nuestro protocolo de tratamiento SSL
            try 
            {
                
                String aux=null;
                String keystorepath="signaturadigital/psisauth.jks";
                String keystorepass="slikZSmz";
                String truststorepath="signaturadigital/cacerts_pre";
                String truststorepass="changeit";

                aux=propFile.getProperty(ConfigProperties.PROP_KEYSTORE_PATH);
                if (aux!=null) keystorepath=aux;
                aux=propFile.getProperty(ConfigProperties.PROP_KEYSTORE_PASSWORD);
                if (aux!=null)keystorepass=aux;
                aux=propFile.getProperty(ConfigProperties.PROP_TRUST_STORE_PATH);
                if (aux!=null) truststorepath=aux;
                aux=propFile.getProperty(ConfigProperties.PROP_TRUST_STORE_PASSWORD);
                if (aux!=null) truststorepass=aux;
                
                @SuppressWarnings("deprecation")
				Protocol authhttps = new Protocol("psishttps",  
                        new AuthSSLProtocolSocketFactory(
                                Thread.currentThread().getContextClassLoader().getResource(keystorepath),keystorepass, //MalformedURLException
                                Thread.currentThread().getContextClassLoader().getResource(truststorepath),truststorepass), 443); //MalformedURLException
                
                Protocol.registerProtocol("psishttps", authhttps);
            } catch (Exception e) {
                log.error(e.getMessage());
            } finally {
                if (httppost!=null) {
                    httppost.releaseConnection();
                }
            }
        }
        
        log.info("BaseSignatura (constructor) done. URL = " + soapURL);  
    }

    protected void iniComunicacio()
    {
        log.info("iniComunicacio() init"); 
        // Configuraci local
        Utils.setProxyConfiguration();
        // Inicialitzaci del client
        digitalSignatureServiceClient proxy = new digitalSignatureServiceClient();
        this.port = proxy.getdssPortSoap(this.soapURL);
        // Definici dels namespaces correctes
        Hashtable<String, String> prefixes = new Hashtable<String, String>();
        prefixes.put(
            propFile.getProperty(ConfigProperties.PROP_PREF1_NAMESPACE_KEY),    //"urn:oasis:names:tc:dss:1.0:core:schema" 
            propFile.getProperty(ConfigProperties.PROP_PREF1_NAMESPACE_VALUE)); //"dss"
        prefixes.put(
            propFile.getProperty(ConfigProperties.PROP_PREF2_NAMESPACE_KEY),    //"http://www.w3.org/2000/09/xmldsig#" 
            propFile.getProperty(ConfigProperties.PROP_PREF2_NAMESPACE_VALUE)); //"ds"
        this.options.setSaveSuggestedPrefixes(prefixes);
        log.info("iniComunicacio() done"); 
    }
    
    protected HashMap<String, String> generaHashMissatges(ResponseBaseType response) {
        
        HashMap<String, String> hashMsg= new HashMap<String, String>();
        
        if (response.getResult().getResultMajor()!=null) {
            hashMsg.put(ConstantsApplicacio.MSG_ResultMajor,response.getResult().getResultMajor());
        }
        if (response.getResult().getResultMinor()!=null) {
            hashMsg.put(ConstantsApplicacio.MSG_ResultMinor,response.getResult().getResultMinor());
        }
        if (response.getResult().getResultMessage()!=null) {
            InternationalStringType inter = response.getResult().getResultMessage();
            inter.setLang("ES");         
            hashMsg.put(ConstantsApplicacio.MSG_ResultMsg,inter.getStringValue());
        }
        if (response.getOptionalOutputs()!=null &&
                response.getOptionalOutputs().getProcessingDetailsArray()!=null) {
            log.debug("procesign details : " + response.getOptionalOutputs().getProcessingDetailsArray()); 
        }
        log.debug(""); 
        log.debug("HASH MSG "+hashMsg);   
        
        return hashMsg;
    }
    
    public String getSoapURL()
    {
        return soapURL;
    }

    public void setSoapURL(String soapURL)
    {
        boolean isHTTPS = soapURL!=null && soapURL.startsWith("https");
        
        // Si queremos hacer una conexin HTTPS, le aadimos el prefijo "psis"
        // para que el keystore y el truststore no afecten a otras conecciones HTTPS
        // dentro de nuestro entorno de trabajo

        this.soapURL = (isHTTPS) ? "psis"+soapURL : soapURL;
    }
    
    public String getProperty(String key)
    {
        return propFile.getProperty(key);
    }
}
