package com.steria.digitalsignature.core;

import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import x0CoreSchema.oasisNamesTcDss1.SignRequestDocument;
import x0CoreSchema.oasisNamesTcDss1.SignResponseDocument;
import x0CoreSchema.oasisNamesTcDss1.SignRequestDocument.SignRequest;
import x0CoreSchema.oasisNamesTcDss1.SignResponseDocument.SignResponse;

import com.steria.digitalsignature.exception.SignaturaExcepcio;
import com.steria.digitalsignature.exception.SignaturaInvalidAttributExcepcio;
import com.steria.digitalsignature.properties.ConfigProperties;
import com.steria.digitalsignature.vo.DigitalSignatureContentVo;

/**
 * @author cvila
 * s la classe de base que heretaran totes les classes daquest projecte que vulguin crear signatures i aquesta heretar les funcions bsiques de BaseSignatura i afegir les especifiques per la creaci de la signatura
	-Instanciar la classe que escriur les sortides de log.
	-Definir atributs necessaris per la creaci de signatura:
	-Definir mtodes abstractes que els seus fills hauran dimplementar:
	    abstract protected void completarVerifyRequest(DigitalSignatureContentVo vo);
	    abstract protected HashMap<String, String> generaHashResultat(SignResponse response);
	-Realitzar les operacions principals que tota classe filla necessitar:
	-Establiment de la comunicaci.
	-Obtenir una estructura DSS de petici de signatura signRequest i iniciarlitzar-lo.
	-Inicialitzaci del Vo amb la informaci principal per poder comenar a treballar.
	-El fet de completar el missatge DSS amb la petici de signatura ser responsabilitat de la classe filla que implementi aquest mtode especfic: completarVerifyRequest(vo);
	-Amb aquesta petici complerta, la classe para Creaci i BaseSignatura realitzaran la connexi al portSoap i endpoint del servei de PSIS per realitzar la petici.
	        log.info("Crida WS PSIS"); 
	  SignResponseDocument responseDocument =   port.sign(this.requestDocument);
	-Obt la resposta del servei i la parsetja o tracta afegint-lo al hash datributs resultants del nostre Vo.
	-I finalment afegeix al Vo un chequeix rpid de si el procs de signatura s correcte o no, mitjanant la variable booleana procesOk on espera que el resultat del missatge dss:ResultMajor sigui el resultat definit com a correcte urn\:oasis\:names \:tc\:dss\:1.0\:resultmajor\:Success (configurable al properties).
 * 
 */
public abstract class Creacio extends BaseSignatura
{
    private static final Log log = LogFactory.getLog(Creacio.class);
    
    protected SignRequestDocument requestDocument = null;
    protected SignRequest signRequest = null;

    // Respuesta esperada en la comunicacin con PSIS
    // En las clases hijas ha de definirse segn corresponda (si cambia)
    // Ej: "urn:oasis:names:tc:dss:1.0:resultmajor:Success"
    protected String resultMajorExpected = "";

    abstract protected void completarVerifyRequest(DigitalSignatureContentVo vo);
    abstract protected HashMap<String, String> generaHashResultat(SignResponse response);
    
    public Creacio() {
        super();
        resultMajorExpected = propFile.getProperty(ConfigProperties.PROP_RESULT_MAJOR_EXPECTED);
    }
    
    public void crear(DigitalSignatureContentVo vo)
    throws SignaturaInvalidAttributExcepcio, SignaturaExcepcio
    {
        log.info("crear() init. [PERFIL: "+vo.getPerfil()+"]");  
        
        iniComunicacio();
        // Creaci del missatge DSS i selecci de profile
        this.requestDocument = SignRequestDocument.Factory.newInstance(options);
        this.signRequest = this.requestDocument.addNewSignRequest();
        if (vo.getPerfil() != null) {
            this.signRequest.setProfile(vo.getPerfil());
        }
        
        // Composici del missatge
        completarVerifyRequest(vo);
        log.info("VerifyRequestDocument:"); 
        log.info(requestDocument);
        log.info("Crida WS PSIS"); 
        SignResponseDocument responseDocument = port.sign(this.requestDocument);
        log.info("VerifyResponseDocument:"); 
        log.info(responseDocument);
        log.info("Segell Creat!"); 
        
        // Tratamos el resultado y lo aadimos al Hash resultante del CertificatVo
        if (responseDocument.getSignResponse().getSignatureObject()!=null)
        {
            byte [] aux=responseDocument.getSignResponse().getSignatureObject().getTimestamp().getSignature().getSignatureValue().getByteArrayValue();
            vo.setSignature(aux);
            
            HashMap<String, String> hashAtributs = generaHashResultat(responseDocument.getSignResponse()); 
            log.info("Hash Atributs: "+hashAtributs); 
            vo.setAtributs(hashAtributs);
        }
        
        HashMap<String, String> hashMsg = new HashMap<String, String>();
        hashMsg = generaHashMissatges(responseDocument.getSignResponse());
        vo.setMsg(hashMsg);
        
        setProcesOk(vo);
        
        log.info("crear() done. [PERFIL: "+vo.getPerfil()+"]");  
    }
    
    public void setProcesOk(DigitalSignatureContentVo vo)
    {
        if (vo.getMsg().get(propFile.getProperty(ConfigProperties.PROP_RESULT_MAJOR)) != null)
        {
            vo.setProcesOk(vo.getMsg().get(propFile.getProperty(ConfigProperties.PROP_RESULT_MAJOR)).equals(resultMajorExpected));
        }
    }
}
