package com.steria.digitalsignature.core;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.XmlAnyURI;
import org.apache.xmlbeans.XmlBase64Binary;
import org.w3.x2000.x09.xmldsig.DigestMethodType;
import org.w3.x2000.x09.xmldsig.KeyInfoType;
import org.w3.x2000.x09.xmldsig.X509DataType;

import x0CoreSchema.oasisNamesTcDss1.DocumentHashDocument.DocumentHash;
import x0CoreSchema.oasisNamesTcDss1.IncludeObjectDocument.IncludeObject;
import x0CoreSchema.oasisNamesTcDss1.InputDocumentsDocument.InputDocuments;
import x0CoreSchema.oasisNamesTcDss1.KeySelectorDocument.KeySelector;
import x0CoreSchema.oasisNamesTcDss1.OptionalInputsDocument.OptionalInputs;
import x0CoreSchema.oasisNamesTcDss1.SignResponseDocument.SignResponse;

import com.steria.digitalsignature.config.ConstantsApplicacio;
import com.steria.digitalsignature.properties.ConfigProperties;
import com.steria.digitalsignature.vo.DigitalSignatureContentVo;

public class CreacioSegellDeTemps extends Creacio
{
    private static final Log log = LogFactory.getLog(CreacioSegellDeTemps.class);
    
    public CreacioSegellDeTemps() {
        super();
        
        log.info("constructor CreacioSegellDeTemps() init"); 
        this.soapURL = this.propFile.getProperty(ConfigProperties.PROP_SOAP_URL);
        log.info("soapURL=["+this.soapURL+"]");  
        log.info("constructor CreacioSegellDeTemps() done"); 
    }
    
    public DigitalSignatureContentVo createDefaultVO() throws Exception {
        
        log.info("createDefaultVO() init"); 
        
        DigitalSignatureContentVo ret = new DigitalSignatureContentVo();
        
        ret.setCertificate(Utils.readBase64File(this.propFile.getProperty(ConfigProperties.PROP_TIMESTAMP_CERTIFICATE_FILE)));
        ret.setData(Utils.readBase64File(this.propFile.getProperty(ConfigProperties.PROP_TIMESTAMP_DIGEST_FILE)));
        ret.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS_Timestmaping);
        
        log.info("createDefaultVO() done"); 
        
        return ret;
    }
    
    protected HashMap<String, String> generaHashResultat(SignResponse response)
    {
        HashMap<String,String> resp=new HashMap<String,String>();

        //Es part de la response.
        if (response.getSignatureObject()!=null && response.getSignatureObject().getTimestamp()!=null)
        {
            ByteArrayOutputStream aux=new ByteArrayOutputStream();
            try
            {
                response.getSignatureObject().getTimestamp().save(aux);
                resp.put(ConstantsApplicacio.TIMESTAMP_signatura,aux.toString("UTF-8"));
            }
            catch (IOException e)
            {
                throw new RuntimeException("Error getting Timestamp"); 
            }
        }
        
        return resp;
    }
    
    protected void completarVerifyRequest (DigitalSignatureContentVo vo)
    {
        try
        {
            if (vo.getCertificate()==null)
            {
                vo.setCertificate(Utils.readBase64File(this.propFile.getProperty(ConfigProperties.PROP_TIMESTAMP_CERTIFICATE_FILE)));
            }
            vo.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS_Timestmaping);
        }
        catch (Exception e)
        {
        }

        // Creaci de l'element amb el document per generar el timestamp
        InputDocuments inpDocuments = InputDocuments.Factory.newInstance();
        DocumentHash document = inpDocuments.addNewDocumentHash();
        document.setID("Doc1"); 
        document.setDigestValue(vo.getData());
        DigestMethodType digestMethod = document.addNewDigestMethod();
        digestMethod.setAlgorithm(this.propFile.getProperty(ConfigProperties.PROP_ALGORITHM_SHA1_URL));        
        this.signRequest.setInputDocuments(inpDocuments);
        // Creaci de l'element amb els parmetres opcionals necessaris per generar el timestamp
        OptionalInputs optional = this.signRequest.addNewOptionalInputs();
        KeySelector selector = optional.addNewKeySelector();
        KeyInfoType key = selector.addNewKeyInfo();
        X509DataType x509data = key.addNewX509Data();
        // S'afegeix el certificat de TSA de CATCert en Base64
        XmlBase64Binary b64certificate = x509data.addNewX509Certificate();
        b64certificate.setByteArrayValue(vo.getCertificate());

        XmlAnyURI any = optional.addNewSignatureType();
        any.setStringValue(ConstantsApplicacio.SCHEMA_XMLtimestamp);
        
        IncludeObject object = optional.addNewIncludeObject();
        object.setObjId("Doc1"); 
        object.setWhichDocument("Doc1"); 
        object.setHasObjectTagsAndAttributesSet(false);
        object.setCreateReference(true);
    }
}
