package com.steria.digitalsignature.core;

import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.XmlDate;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlInteger;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlString;

import x0CoreSchema.oasisNamesTcDss1.VerifyRequestDocument;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument;
import x0CoreSchema.oasisNamesTcDss1.VerifyRequestDocument.VerifyRequest;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument.VerifyResponse;
import x0ProfilesXSS.oasisNamesTcDss1.X509CertificateInfoDocument.X509CertificateInfo;

import com.steria.digitalsignature.exception.SignaturaExcepcio;
import com.steria.digitalsignature.exception.SignaturaInvalidAttributExcepcio;
import com.steria.digitalsignature.properties.ConfigProperties;
import com.steria.digitalsignature.vo.DigitalSignatureContentVo;

/**
 * @author cvila
 * s la classe de base que heretaran totes les classes daquest projecte que vulguin validar certificats, signatures o segells de temps i aquesta heretar les funcions bsiques de BaseSignatura i afegir les especifiques per la validacions. Aquesta classe funciona de la mateixa forma que la clase de Creaci i practicament realitza les mateixes accions amb petites variacions com que el missatge DSS que genera s de verifyRequest (de verificaci de signatura en comptes de signRequest, creaci de signatura). Per aix no entrarem en ms detall daquesta clase.
 *
 *	-Instanciar la classe que escriur les sortides de log.
	-Definir atributs necessaris per la creaci de signatura:
	-Definir mtodes abstractes que els seus fills hauran dimplementar:
	    abstract protected void completarVerifyRequest(DigitalSignatureContentVo vo);
	    abstract protected HashMap<String, String> generaHashResultat(SignResponse response);
	-Realitzar les operacions principals que tota classe filla necessitar:
	-Establiment de la comunicaci.
	-Obtenir una estructura DSS de petici de signatura signRequest i iniciarlitzar-lo.
	-Inicialitzaci del Vo amb la informaci principal per poder comenar a treballar.
	-El fet de completar el missatge DSS amb la petici de signatura ser responsabilitat de la classe filla que implementi aquest mtode especfic: completarVerifyRequest(vo);
	-Amb aquesta petici complerta, la classe para Creaci i BaseSignatura realitzaran la connexi al portSoap i endpoint del servei de PSIS per realitzar la petici.
	        log.info("Crida WS PSIS"); 
	  SignResponseDocument responseDocument =   port.sign(this.requestDocument);
	-Obt la resposta del servei i la parsetja o tracta afegint-lo al hash datributs resultants del nostre Vo.
	-I finalment afegeix al Vo un chequeix rpid de si el procs de signatura s correcte o no, mitjanant la variable booleana procesOk on espera que el resultat del missatge dss:ResultMajor sigui el resultat definit com a correcte urn\:oasis\:names \:tc\:dss\:1.0\:resultmajor\:Success (configurable al properties).
 *
 */
public abstract class Validacio extends BaseSignatura
{
    private static final Log log = LogFactory.getLog(Validacio.class);
    
    protected VerifyRequestDocument requestDocument = null;
    protected VerifyRequest verifyRequest = null;
    
    // Respuesta esperada en la comunicacin con PSIS
    // En las clases hijas ha de definirse segn corresponda (si cambia)
    // Ej: "urn:oasis:names:tc:dss:1.0:resultmajor:Success"
    protected String resultMajorExpected = "";
    
    abstract protected void completarVerifyRequest(DigitalSignatureContentVo vo) throws XmlException;
    abstract protected HashMap<String, String> generaHashResultat(VerifyResponse response);
    
    public Validacio() {
        super();
        resultMajorExpected = propFile.getProperty(ConfigProperties.PROP_RESULT_MAJOR_EXPECTED);
    }
    
    public void validar(DigitalSignatureContentVo vo)
    throws SignaturaInvalidAttributExcepcio, SignaturaExcepcio, XmlException
    {
        log.info("validar() init. [PERFIL: "+vo.getPerfil()+"]");  
        iniComunicacio();
        // Creaci del missatge DSS i selecci de profile
        this.requestDocument = VerifyRequestDocument.Factory.newInstance(options);
        this.verifyRequest = this.requestDocument.addNewVerifyRequest();
        if (vo.getPerfil() != null) {
            this.verifyRequest.setProfile(vo.getPerfil());
        }
        // Composici del missatge
        completarVerifyRequest(vo);
        log.info("VerifyRequestDocument:"); 
        log.info(requestDocument);
        log.info("Crida WS PSIS"); 
        VerifyResponseDocument responseDocument = this.port.verify(requestDocument);
        log.info("VerifyResponseDocument:"); 
        log.info(responseDocument);
        log.info("Validaci realitzada, resposta rebuda."); 

        // Tratamos el resultado y lo aadimos al Hash resultante del CertificatVo
        HashMap<String, String> hashAtributs = generaHashResultat(responseDocument.getVerifyResponse()); 
        log.info("Hash Atributs: "+hashAtributs); 
        vo.setAtributs(hashAtributs);
        HashMap<String, String> hashMsg = new HashMap<String, String>();
        hashMsg = generaHashMissatges(responseDocument.getVerifyResponse()); 
        vo.setMsg(hashMsg);
        log.info("Hash Msg(returned): "+hashMsg); 
        setProcesOk(vo);
        log.info("validar() done. [PERFIL: "+vo.getPerfil()+"]");  
    }
    
    public void setProcesOk(DigitalSignatureContentVo vo)
    {
        if (vo.getMsg().get(propFile.getProperty(ConfigProperties.PROP_RESULT_MAJOR)) != null)
        {
            vo.setProcesOk(vo.getMsg().get(propFile.getProperty(ConfigProperties.PROP_RESULT_MAJOR)).equals(resultMajorExpected));
        }
        if (vo.isProcesOk())
        {
            //Las validaciones de certificados son incorrectas is se recibe un minor o este
            // no es el de todas las firmas correctas.
            if(vo.getMsg().get("dss:ResultMinor")==null || vo.getMsg().get("dss:ResultMinor").indexOf(":valid:")==-1)
            {
                vo.setProcesOk(false);
            }
        }
    }
    
  
}
