package com.steria.digitalsignature.core;

import java.util.Arrays;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.XmlDate;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlInteger;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlString;
import org.w3.x2000.x09.xmldsig.DigestMethodType;

import x0CoreSchema.oasisNamesTcDss1.DocumentHashDocument.DocumentHash;
import x0CoreSchema.oasisNamesTcDss1.InputDocumentsDocument.InputDocuments;
import x0CoreSchema.oasisNamesTcDss1.OptionalInputsDocument.OptionalInputs;
import x0CoreSchema.oasisNamesTcDss1.SignatureObjectType;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument.VerifyResponse;
import x0ProfilesXSS.oasisNamesTcDss1.SignatureInfoDocument.SignatureInfo;

import com.steria.digitalsignature.config.ConstantsApplicacio;
import com.steria.digitalsignature.properties.ConfigProperties;
import com.steria.digitalsignature.vo.DigitalSignatureContentVo;

/**
 * @author cvila
 * Aquesta classe es centrar en la lgica de negoci prpia per realitzar la petici de validaci de segell de temps. Aquest cas s un servei diferent que ofereix CATCert que permet validar segell de temps, per aix utilitzarem el perfil diferent: urn:oasis:names:tc:dss:1.0:profiles:timestamping.
 */
public class ValidacioSegellDeTemps extends Validacio
{
    private static final Log log = LogFactory.getLog(ValidacioSegellDeTemps.class);
    
    public ValidacioSegellDeTemps()
    {
        super();
        
        log.info("constructor ValidacioSegellDeTemps() init"); 
        this.soapURL = propFile.getProperty(ConfigProperties.PROP_SOAP_URL);
        log.info("soapURL=["+this.soapURL+"]");  
        log.info("constructor ValidacioSegellDeTemps() done"); 
    }
    
    public DigitalSignatureContentVo createDefaultVO() throws Exception
    {
        
        log.info("createDefaultVO() init"); 

        DigitalSignatureContentVo ret = new DigitalSignatureContentVo();
        
        ret.setData(Utils.readBase64File(propFile.getProperty(ConfigProperties.PROP_TIMESTAMP_DIGEST_FILE)));
        ret.setInlineXML(Utils.readXmlFile(propFile.getProperty(ConfigProperties.PROP_TIMESTAMP_DATA_FILE)));
        ret.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS_Timestmaping);
        ret.getAttributsOpcionals().addAll(Arrays.asList(ConstantsApplicacio.SignatureCMSAtributsDefecte));
        
        log.info("createDefaultVO() done"); 
        
        return ret;        
    }
    
    /* 
     * Obt la resposta del servei i la parsetja o tracta afegint-lo al hash datributs resultants del nostre Vo. Un hashmap datributs on la key ser el nom dels atributs que ens interessen en el procs i el valor el contingut del atribut retornat. Per exemple: DigitalSignatureContentVo.atributs.get(ApplicationConfig.NIF) retornar el valor del NIF obtingut a la validaci. Els atributs obligatoris sempre vindran complimentats o amb valor buit i els atributs opcionals safegiran posteriorment.
     */
    protected HashMap<String, String> generaHashResultat(VerifyResponse response) {
        
        HashMap<String, String> hashAtributs = new HashMap<String, String>();
        
        if (response.getOptionalOutputs()!=null && response.getOptionalOutputs().getSignatureInfoArray().length>0) {
            SignatureInfo info = response.getOptionalOutputs().getSignatureInfoArray()[0];
            for (int i=0; i<info.sizeOfAttributeArray(); i++) {
                XmlObject[] values = info.getAttributeArray(i).getAttributeValueArray();
                if (values.length > 0) {
                    XmlObject xmlObject = values[0];
                    String keyName = info.getAttributeArray(i).getName();
                    log.debug("xmlObject: " + xmlObject); 
                    if (xmlObject!=null) {
                        if(xmlObject instanceof XmlDate) {
                            XmlDate xmlDate = (XmlDate) xmlObject;
                            hashAtributs.put(keyName,xmlDate.getStringValue());
                            log.debug("Date: " + xmlDate.getDateValue());                     
                        }
                        else if(xmlObject instanceof XmlString) {
                            XmlString xmlString = (XmlString) xmlObject;
                            hashAtributs.put(keyName,xmlString.getStringValue());
                            log.debug("String: " + xmlString.getStringValue()); 
                        }
                        else if(xmlObject instanceof XmlInteger) {
                            XmlInteger xmlInteger = (XmlInteger) xmlObject;
                            hashAtributs.put(keyName,xmlInteger.getStringValue());
                            log.debug("Integer: " + xmlInteger.getStringValue()); 
                        }
                    }
                }
            }
        }
        
        return hashAtributs;
    }
    
    /* 
     * Completar el missatge DSS de validaci de segell de temps verifyRequest i el DigitalSignatureContentVo a travs de la implementaci daquest mtode especfic. En aquest sagafa la informaci del Vo com la signatura i atributs addicionals i s crea la estructura oportuna del missatge DSS de petici de validaci segell de temps segons la descripci feta daquesta en lapartat de missatges DSS. 
     */
    protected void completarVerifyRequest(DigitalSignatureContentVo vo) throws XmlException
    {

        try
        {
            if (vo.getSignature()==null)
            {
                vo.setSignature(Utils.readBase64File(propFile.getProperty(ConfigProperties.PROP_TIMESTAMP_DIGEST_FILE)));
            }
            vo.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS_Timestmaping);
        }
        catch (Exception e)
        {
        }
        vo.getAttributsOpcionals().addAll(Arrays.asList(ConstantsApplicacio.SignatureCMSAtributsDefecte));
        
        SignatureObjectType signaturetype = SignatureObjectType.Factory.parse(vo.getInlineXML(),options);
        this.verifyRequest.setSignatureObject(signaturetype);
        // Creaci de l'element amb el document per generar el timestamp
        InputDocuments inpDocuments = this.verifyRequest.addNewInputDocuments();
        DocumentHash document = inpDocuments.addNewDocumentHash();
        document.setID("Doc1"); 
        document.setDigestValue(vo.getData());  //igual que el creaciSegellDeTemps
        DigestMethodType digestMethod = document.addNewDigestMethod();
        digestMethod.setAlgorithm(this.propFile.getProperty(ConfigProperties.PROP_ALGORITHM_SHA1_URL));        
        // Creaci de l'element amb els parmetres opcionals necessaris per generar el timestamp
        OptionalInputs optional = this.verifyRequest.addNewOptionalInputs();
        optional.addNewReturnProcessingDetails();
    }        
}
