package com.steria.digitalsignature.core;

import java.util.Arrays;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.XmlDate;
import org.apache.xmlbeans.XmlInteger;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlString;
import org.w3.x2000.x09.xmldsig.DigestMethodType;

import x0CoreSchema.oasisNamesTcDss1.DocumentType;
import x0CoreSchema.oasisNamesTcDss1.SignatureObjectType;
import x0CoreSchema.oasisNamesTcDss1.Base64DataDocument.Base64Data;
import x0CoreSchema.oasisNamesTcDss1.Base64SignatureDocument.Base64Signature;
import x0CoreSchema.oasisNamesTcDss1.DocumentHashDocument.DocumentHash;
import x0CoreSchema.oasisNamesTcDss1.InputDocumentsDocument.InputDocuments;
import x0CoreSchema.oasisNamesTcDss1.OptionalInputsDocument.OptionalInputs;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument.VerifyResponse;
import x0ProfilesXSS.oasisNamesTcDss1.ReturnSignatureInfoDocument.ReturnSignatureInfo;
import x0ProfilesXSS.oasisNamesTcDss1.SignatureInfoDocument.SignatureInfo;

import com.steria.digitalsignature.config.ConstantsApplicacio;
import com.steria.digitalsignature.exception.SignaturaInvalidAttributExcepcio;
import com.steria.digitalsignature.properties.ConfigProperties;
import com.steria.digitalsignature.vo.DigitalSignatureContentVo;

/**
 * @author cvila
 * Aquestes classes hereten de les pare que les aporten tota la infraestructura necessria per treballar i per tant la classe ValidacioSignatura es centrar en la lgica de negoci prpia per realitzar la petici de validaci duna signatura de tipus CMS detached, ja sigui una signatura CMS detached amb el document, sense o b una signatura CMS detached per del resum del document (del hash del document)
 */
public class ValidacioSignatura extends Validacio
{
    private static final Log log = LogFactory.getLog(ValidacioSignatura.class);
    
    public static enum TipusSignatura {CMSdetached, CMSdetachedDoc, CMSdetachedHashDoc};
    
    protected TipusSignatura tipusSignatura;
    
    public TipusSignatura getTipusSignatura()
    {
        return tipusSignatura;
    }
    
    public ValidacioSignatura(TipusSignatura tipusSignatura)
    {
        super();
        log.info("constructor ValidacioSignatura() init"); 
        this.soapURL = propFile.getProperty(ConfigProperties.PROP_SOAP_URL);
        log.info("soapURL=["+this.soapURL+"]");  
        this.tipusSignatura = tipusSignatura;
        log.info("constructor ValidacioSignatura() done"); 
    }
    
    /**
     * @return
     * El mtode que crea un Vo per defecte que obt informaci de prova, com un signatures de prova per realitzar la petici de validaci duna signatura de tipus CMS detached, ja sigui una signatura CMS detached amb el document, sense o b una signatura CMS detached per del resum del document (del hash del document) del fitxer de propietats. En un cas real aquest mtode no sutilitza i es carreguen dades reals de les signatures a traves dels mtodes i atributs del DigitalSignatureContentVo.
     * @throws Exception
     */
    public DigitalSignatureContentVo createDefaultVO() throws Exception
    {
        
        log.info("createDefaultVO() init"); 

        DigitalSignatureContentVo ret = new DigitalSignatureContentVo();
        /*
        if (this.tipusSignatura == TipusSignatura.CMSdetached) {
            ret.setSignature(Utils.readBase64File(propFile.getProperty(ConfigProperties.PROP_SIGNATURE_CMS_FILE)));
            ret.setData(Utils.readBase64File(propFile.getProperty(ConfigProperties.PROP_SIGNED_CMS_FILE)));
        } else if (this.tipusSignatura == TipusSignatura.CMSdetachedDoc) {
            ret.setSignature(Utils.readBase64File(propFile.getProperty(ConfigProperties.PROP_SIGNATURE_CMS_DETACHED_FILE)));
            ret.setData(Utils.readBase64File(propFile.getProperty(ConfigProperties.PROP_DATA_CMS_DETACHED_FILE)));
        } else if (this.tipusSignatura == TipusSignatura.CMSdetachedHashDoc) {
            ret.setSignature(Utils.readBase64File(propFile.getProperty(ConfigProperties.PROP_SIGNATURE_CMS_DETACHED_HASH_FILE)));
            ret.setData(Utils.readBase64File(propFile.getProperty(ConfigProperties.PROP_DATA_CMS_DETACHED_HASH_FILE)));
        }*/
        
        ret.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS);
        //ret.getAttributsOpcionals().addAll(Arrays.asList(ConstantsApplicacio.SignatureCMSAtributsDefecte));
        
        log.info("createDefaultVO() done"); 
        
        return ret;        
    }
    /* 
     * Obt la resposta del servei i la parsetja o tracta afegint-lo al hash datributs resultants del nostre Vo. Un hashmap datributs on la key ser el nom dels atributs que ens interessen en el procs i el valor el contingut del atribut retornat. Per exemple: DigitalSignatureContentVo.atributs.get(ApplicationConfig.NIF) retornar el valor del NIF obtingut a la validaci. Els atributs obligatoris sempre vindran complimentats o amb valor buit i els atributs opcionals safegiran posteriorment.
     */    
    /* (non-Javadoc)
     * @see com.steria.digitalsignature.core.Validacio#generaHashResultat(x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument.VerifyResponse)
     */
    protected HashMap<String, String> generaHashResultat(VerifyResponse response) {
        
        HashMap<String, String> hashAtributs = new HashMap<String, String>();
        
        if (response.getOptionalOutputs()!=null && response.getOptionalOutputs().getSignatureInfoArray().length>0) {
            SignatureInfo info = response.getOptionalOutputs().getSignatureInfoArray()[0];
            for (int i=0; i<info.sizeOfAttributeArray(); i++) {
                XmlObject[] values = info.getAttributeArray(i).getAttributeValueArray();
                if (values.length > 0) {
                    XmlObject xmlObject = values[0];
                    String keyName = info.getAttributeArray(i).getName();
                    log.debug("xmlObject: " + xmlObject); 
                    if (xmlObject!=null) {
                        if(xmlObject instanceof XmlDate) {
                            XmlDate xmlDate = (XmlDate) xmlObject;
                            hashAtributs.put(keyName,xmlDate.getStringValue());
                            log.debug("Date: " + xmlDate.getDateValue());                     
                        }
                        else if(xmlObject instanceof XmlString) {
                            XmlString xmlString = (XmlString) xmlObject;
                            hashAtributs.put(keyName,xmlString.getStringValue());
                            log.debug("String: " + xmlString.getStringValue()); 
                        }
                        else if(xmlObject instanceof XmlInteger) {
                            XmlInteger xmlInteger = (XmlInteger) xmlObject;
                            hashAtributs.put(keyName,xmlInteger.getStringValue());
                            log.debug("Integer: " + xmlInteger.getStringValue()); 
                        }
                    }
                }
            }
        }
        
        return hashAtributs;
    }
    
    /* 
     * Completar el missatge DSS de petici de signatura verifyRequest i el DigitalSignatureContentVo a travs de la implementaci daquest mtode especfic. En aquest sagafa la informaci del Vo com el certificat i atributs addicionals i s crea la estructura oportuna del missatge DSS de petici de signatura segons la descripci feta daquesta en lapartat de missatges DSS
     * Podem veure que en aquest mtode afegim el tipus de perfil del protocol que volem utilitzar, en aquest cas el perfil estes de CATCert XSS. 
     * vo.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS);
     * 
		Tamb afegim a la llista datributs els atributs per defecte daquest tipus de validaci CMS
		vo.getAttributsOpcionals().addAll(Arrays.asList(ConstantsApplicacio.SignatureCMSAtributsDefecte));
		Aqu creem la resta de lestructura del missatge DSS de validaci de signatures CMS, demanant les opcions optatives i la resta datributs opcionals que vulguem per loperaci.  Aquesta estructura s la del cas SignatureObject. Base64Signature que varem veure a captol de missatges DSS.
		Desprs comprovant quina de les signatures CMS estem utilitzant inclourem el hash del document, el document o simplement la signatura CMS en el missatge DSS de validaci.

     */
    protected void completarVerifyRequest(DigitalSignatureContentVo vo)
    {
        try
        {
            vo.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS);
            vo.getAttributsOpcionals().addAll(Arrays.asList(ConstantsApplicacio.SignatureCMSAtributsDefecte));
        }
        catch (SignaturaInvalidAttributExcepcio e)
        {
        }

        // Generamos el tag OptionalInputs y dems atributos
        OptionalInputs optionalInputs = this.verifyRequest.addNewOptionalInputs();
        optionalInputs.addNewReturnProcessingDetails();
        if (vo.getAttributsOpcionals().size()>0) {
            ReturnSignatureInfo returnSignInfo = optionalInputs.addNewReturnSignatureInfo();
            for (String atribut : vo.getAttributsOpcionals()) {
                returnSignInfo.addNewAttributeDesignator().setName(atribut);
            }        
            optionalInputs.setReturnSignatureInfoArray( new ReturnSignatureInfo[] { returnSignInfo } );
        }
        // Generamos el tag SignatureObject     
        SignatureObjectType signatureObject = this.verifyRequest.addNewSignatureObject();
        // Generamos el tag Base64Signature     
        Base64Signature base64Signature = signatureObject.addNewBase64Signature();
        base64Signature.setByteArrayValue(vo.getSignature());
        base64Signature.setType(propFile.getProperty(ConfigProperties.PROP_BASE64_CMS_SIGNATURA_TYPE)); //"urn:ietf:rfc:3852"
        signatureObject.setBase64Signature(base64Signature);
        // Com es deatached hem d'afegir el document per la validaci amb la signatura del hash
        InputDocuments inputDocuments = this.verifyRequest.addNewInputDocuments();
        if (this.tipusSignatura == TipusSignatura.CMSdetachedHashDoc)
        {
            DocumentHash hashDoc = inputDocuments.addNewDocumentHash();
            DigestMethodType digestMethod = hashDoc.addNewDigestMethod();
            digestMethod.setAlgorithm(propFile.getProperty(ConfigProperties.PROP_ALGORITHM_SHA1_URL)); //"http://www.w3.org/2000/09/xmldsig#sha1";
            hashDoc.setDigestValue(vo.getData());
        }
        else if (this.tipusSignatura == TipusSignatura.CMSdetached || this.tipusSignatura == TipusSignatura.CMSdetachedDoc)
        {
            DocumentType docType = inputDocuments.addNewDocument();
            Base64Data document = docType.addNewBase64Data();
            document.setByteArrayValue(vo.getData());
        }
    }    
}
