package com.steria.digitalsignature.core;

import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.XmlDate;
import org.apache.xmlbeans.XmlInteger;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlString;

import x0Assertion.oasisNamesTcSAML1.NameIdentifierType;
import x0CoreSchema.oasisNamesTcDss1.DocumentType;
import x0CoreSchema.oasisNamesTcDss1.Base64DataDocument.Base64Data;
import x0CoreSchema.oasisNamesTcDss1.InputDocumentsDocument.InputDocuments;
import x0CoreSchema.oasisNamesTcDss1.OptionalInputsDocument.OptionalInputs;
import x0CoreSchema.oasisNamesTcDss1.VerifyResponseDocument.VerifyResponse;
import x0ProfilesXSS.oasisNamesTcDss1.X509CertificateInfoDocument.X509CertificateInfo;

import com.steria.digitalsignature.config.ConstantsApplicacio;
import com.steria.digitalsignature.exception.SignaturaInvalidAttributExcepcio;
import com.steria.digitalsignature.properties.ConfigProperties;
import com.steria.digitalsignature.vo.DigitalSignatureContentVo;

/**
 * @author cvila
 * Aquestes classes hereten de les pare que les aporten tota la infraestructura necessria per treballar i per tant la classe ValidacioSignatura es centrar en la lgica de negoci prpia per realitzar la petici de validaci de signatures de PDF. Aquest cas s un servei diferent que ofereix CATCert que permet validar signatures de PDF amb particularitats, com afegir ra de signatura i daltres, per aix utilitzarem el perfil diferent: urn:oasis:names:tc:dss:1.0:profiles:DSS_PDF. 
 */
public class ValidacioSignaturaPDF extends Validacio
{
    private static final Log log = LogFactory.getLog(ValidacioSignaturaPDF.class);
    
    public ValidacioSignaturaPDF() {
        super();
        
        log.info("constructor ValidacioSignaturaPDF() init"); 
        this.setSoapURL(propFile.getProperty(ConfigProperties.PROP_SOAP_DSS_PDF_URL)); //"http://psisbeta.catcert.net/psis/catcert-test/dsspdf"
        log.info("soapURL=["+this.soapURL+"]");  
        log.info("constructor ValidacioSignaturaPDF() done"); 
    }
    
    public DigitalSignatureContentVo createDefaultVO() throws Exception {
        log.info("createDefaultVO() init"); 
        DigitalSignatureContentVo ret = new DigitalSignatureContentVo();
        ret.setData(Utils.getBytesFromFile(propFile.getProperty(ConfigProperties.PROP_SIGNED_PDF_FILE)));
        ret.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS_PDF);
        log.info("createDefaultVO() done"); 
        return ret;
    }

    
    /* 
     * Obt la resposta del servei i la parsetja o tracta afegint-lo al hash datributs resultants del nostre Vo. Un hashmap datributs on la key ser el nom dels atributs que ens interessen en el procs i el valor el contingut del atribut retornat. Per exemple: DigitalSignatureContentVo.atributs.get(ApplicationConfig.NIF) retornar el valor del NIF obtingut a la validaci. Els atributs obligatoris sempre vindran complimentats o amb valor buit i els atributs opcionals safegiran posteriorment.
     */
    protected HashMap<String, String> generaHashResultat(VerifyResponse response) {
        HashMap<String, String> hashAtributs = new HashMap<String, String>();
        
        if (response.getOptionalOutputs()!=null &&
                response.getOptionalOutputs().getX509CertificateInfoArray().length>0) {
            X509CertificateInfo x509CertificateInfo = response.getOptionalOutputs().getX509CertificateInfoArray()[0];
            for (int i=0; i<x509CertificateInfo.sizeOfAttributeArray(); i++) {
                XmlObject[] values = x509CertificateInfo.getAttributeArray(i).getAttributeValueArray();
                if (values.length > 0) {
                    XmlObject xmlObject = values[0];
                    String keyName = x509CertificateInfo.getAttributeArray(i).getName();
                    log.debug("xmlObject: " + xmlObject); 
                    if (xmlObject!=null) {
                        if(xmlObject instanceof XmlDate) {
                            XmlDate xmlDate = (XmlDate) xmlObject;
                            hashAtributs.put(keyName,xmlDate.getStringValue());
                            log.debug("Date: " + xmlDate.getDateValue());                     
                        }
                        else if(xmlObject instanceof XmlString) {
                            XmlString xmlString = (XmlString) xmlObject;
                            hashAtributs.put(keyName,xmlString.getStringValue());
                            log.debug("String: " + xmlString.getStringValue()); 
                        }
                        else if(xmlObject instanceof XmlInteger) {
                            XmlInteger xmlInteger = (XmlInteger) xmlObject;
                            hashAtributs.put(keyName,xmlInteger.getStringValue());
                            log.debug("Integer: " + xmlInteger.getStringValue()); 
                        }
                    }
                }
            }
        }
        
        return hashAtributs;
    }
    /*    
    protected HashMap<String, String> generaHashResultat(VerifyResponse response) {
        
        HashMap<String, String> hashAtributs = new HashMap<String, String>();
        
        if (response.getOptionalOutputs()!=null) 
        {
            NameIdentifierType rsi = response.getOptionalOutputs().getSignerIdentityArray(0);
            if (rsi!=null)
            {
                String[] aValues = rsi.getStringValue().split(","); 
                for (int i=0; i<aValues.length; i++)
                {
                    String[] aPair = aValues[i].split("="); 
                    String key = aPair[0];
                    String value = aPair[1];
                    
                    // Por si se repite la key
                    if (hashAtributs.get(key)!=null)
                    {
                        // Le adicionamos un salto de lnea y aadimos el nuevo valor
                        hashAtributs.put(key, hashAtributs.get(key)+"\\n"+value); 
                    }
                    else
                    {
                        hashAtributs.put(key, value);
                    }
                }
            }
        }
        
        return hashAtributs;
    }*/
    
    /* 
     * Completar el missatge DSS de validaci de signatura PDF verifyRequest i el DigitalSignatureContentVo a travs de la implementaci daquest mtode especfic. En aquest sagafa la informaci del Vo com la signatura i atributs addicionals i s crea la estructura oportuna del missatge DSS de petici de validaci de signatura PDF segons la descripci feta daquesta en lapartat de missatges DSS. 
     */
    protected void completarVerifyRequest(DigitalSignatureContentVo vo)
    {
        try
        {
            vo.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS_PDF);
        }
        catch (SignaturaInvalidAttributExcepcio e)
        {
        }

        // Generamos el tag OptionalInputs y dems atributos
        OptionalInputs optionalInputs = this.verifyRequest.addNewOptionalInputs();
        optionalInputs.addNewReturnSignatureReason();
        optionalInputs.addNewReturnSignerIdentity();
        
        //optionalInputs.addNewReturnSignatureReason();
        // Com es deatached hem d'afegir el document per la validaci amb la signatura del hash
        InputDocuments inputDocuments = this.verifyRequest.addNewInputDocuments();
        DocumentType documentLlegit = inputDocuments.addNewDocument();
        // Llegim document de disc
        Base64Data base64Document = Base64Data.Factory.newInstance();
        base64Document.setByteArrayValue(vo.getData());
        documentLlegit.setBase64Data(base64Document);
    }
}
