package com.steria.digitalsignature.core;

import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import junit.framework.TestCase;

import com.steria.digitalsignature.config.ConstantsApplicacio;
import com.steria.digitalsignature.properties.ConfigProperties;
import com.steria.digitalsignature.core.ValidacioSignatura.TipusSignatura;
import com.steria.digitalsignature.vo.DigitalSignatureContentVo;

/**
 * @author cvila
 * Paquet que cont el nucli de la lgica de negoci del testeig.
 */
public class SignaturaTest extends TestCase
{
    protected ConfigProperties propFile = null;
    private static final Log log = LogFactory.getLog(SignaturaTest.class);

    protected void setUp() throws Exception
    {
        propFile = ConfigProperties.getInstance(true);        
    }

    protected void tearDown() throws Exception
    {
    }
    
    /**
     * Prova amb certificat digital correcte
     *  Aquest valida un certificat X509 de manera que comprova que el certificat sigui vlid, la persona s qui diu ser, no ha estat.
		Parmetres entrada: En aquesta prova passem com a parmetre dentrada el certificat personal didentificaci: signaturadigital/cert/CARLOSVILAMATEOS.crt
		Resultat: Finalment retornar el objecte DigitalSignatureContentVo amb els atributs de sortida:
		- Un hashmap datributs on la key ser el nom dels atributs que ens interessen en el procs i el valor el contingut del atribut retornat. Per exemple: DigitalSignatureContentVo.atributs.get(ApplicationConfig.NIF) retornar el valor del NIF obtingut a la validaci. Els atributs obligatoris sempre vindran complimentats o amb valor buit i els atributs opcionals safegiran posteriorment.
		    	protected HashMap<?, ?> atributs;     		
		- Un camp de resultat boole que indicar si el procs s correcte (true  sha validat positivament la signatura) o ha fallat (false  no sha validat la signatura).  Pot ser correcte si es valida el certificat, si es valida la signatura, si es retorna segell de temps o pot se fals si dona qualsevol excepci (no hi ha connexi, etc.) o no es valida el certificat, no es valida la signatura, etc.
		   	protected boolean procesOk;		
		- Un hashmap on afegirem tota la informaci de validaci i detalls de sortida del procs.
		    	protected HashMap<String, String> msg;  		
		Per verificar que el test es satisfactori, ho comprovarem a travs dassertions que el missatge de detall del resultat de la validaci en el format DSS sigui igual a lexpressi que indica que el certificat es valid:  				vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
		      assertTrue(minor.contains("valid:certificate:Definitive"));     
     * @throws Exception
     */
    public void testX509Certificate() throws Exception
    {
        try
        {
            Validacio val = new ValidacioCertificat();
            DigitalSignatureContentVo vo = ((ValidacioCertificat)val).createDefaultVO(); 
            String certFile = propFile.getProperty(ConfigProperties.PROP_CERT_FILE);
            log.info("testing certFile=["+certFile+"]");               
            vo.setCertificate(Utils.getBytesFromFile(certFile));           
            val.validar(vo); // cridar a validar classe pare i per lherncia i polimorfisme acabar cridant als mtodes especfics de ValidacioSignatura           
            String minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
            assertTrue(minor.contains("valid:certificate:Definitive"));                        
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(); 
        }
    }

    /**
     * Prova amb certificat digital correcte i extraent atributs del certificat com el NIF
     * Aquest valida un certificat X509 de manera que comprova que el certificat sigui vlid, la persona s qui diu ser, no ha estat revocat. Per a banda demanem les funcionalitats exteses de la normalitzaci datributs del certificat i per tant que ens retorni els atributs que demanem com per exemple al Nif amb el que podrem associar al login de la plataforma juntament amb el certificat digital i aix realitzar una validaci SSO amb certificat.
		Parmetres entrada: En aquesta prova passem com a parmetre dentrada el certificat personal didentificaci: signaturadigital/cert/CARLOSVILAMATEOS.crt
		Resultat: El resultat es guardar als mateixos objectes que els definits en la resta de test al objecte DigitalSignatureContentVo.		
		Per verificar que el test es satisfactori, ho comprovarem a travs dassertions, en aquest cas comprovarem que el missatge de detall del resultat de la validaci en el format DSS sigui igual a lexpressi que indica que el certificat es valid:  				
			  vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
		      assertTrue(minor.contains("valid:certificate:Definitive"));                        
		I tamb comprovarem que ens han retornat atributs com el NIF:
		String nif =(String)(vo.getAtributs().get(ConstantsApplicacio.CERT_NIF));
		assertTrue(nif.contains("46358509P"));
     * @throws Exception
     */
    public void testX509CertificateInfo() throws Exception
    {
        try
        {
            Validacio val = new ValidacioCertificat();
            DigitalSignatureContentVo vo = ((ValidacioCertificat)val).createDefaultVO(); 
            String certFile = propFile.getProperty(ConfigProperties.PROP_CERT_FILE);
            log.info("testing certFile=["+certFile+"]");               
            vo.setCertificate(Utils.getBytesFromFile(certFile));
            // afegim els atributs per defecte per un X509Certificat  - convertir arrays en lista
            List<String> attributsOpcionals = new ArrayList<String> ();
            attributsOpcionals.addAll(Arrays.asList(ConstantsApplicacio.X509CertificateAtributsDefecte));
            vo.setAttributsOpcionals(attributsOpcionals);
            // afegim operacions per defecte per un X509Certificat
            List<String> operacionsAddicionals = new ArrayList<String> ();
            operacionsAddicionals.addAll(Arrays.asList(ConstantsApplicacio.X509CertificateOperacionsDefecte));
            vo.setOperacionsAddicionals(operacionsAddicionals);
            
            val.validar(vo); // cridar a validar classe pare i per lherncia i polimorfisme acabar cridant als mtodes especfics de ValidacioSignatura           
            String nif = (String) (vo.getAtributs().get(ConstantsApplicacio.CERT_NIF));
            assertTrue(nif.contains("46358509P"));
            String minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
            assertTrue(minor.contains("valid:certificate:Definitive"));                        
            
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(); 
        }
    }
    
    /**
     * Prova amb certificat digital expirat
     * Aquest valida un certificat X509 de manera que comprova que el certificat sigui vlid, la persona s qui diu ser, no ha estat revocat. En aquest cas comprovem que es detecta un cas de certificat que va expirar i per tant invalid.
     * Parmetres entrada: En aquesta prova passem com a parmetre dentrada el certificat personal didentificaci expirat: signaturadigital/cert/cpisr_griselda.crt 
     * Per verificar que el test es satisfactori, ho comprovarem a travs dassertions, en aquest cas comprovarem que el missatge de detall del resultat de la validaci en el format DSS sigui igual a lexpressi que indica que el certificat est revocat:  				vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);  
		assertTrue(minor.contains("invalid:certificate:Expired"));
     * @throws Exception
     */
    public void testX509CertificateExpired() throws Exception
    {
        try
        {
            Validacio val = new ValidacioCertificat();
            DigitalSignatureContentVo vo = ((ValidacioCertificat)val).createDefaultVO(); 
            String certFile = propFile.getProperty(ConfigProperties.PROP_CERT_FILE_EXPIRED);
            log.info("testing certFile=["+certFile+"]");               
            vo.setCertificate(Utils.getBytesFromFile(certFile));           
            val.validar(vo); // cridar a validar classe pare i per lherncia i polimorfisme acabar cridant als mtodes especfics de ValidacioSignatura 
            String minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
            assertTrue(minor.contains("invalid:certificate:Expired"));                        
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(); 
        }
    }

    /**
     * Prova amb certificat digital revocat
     * Aquest valida un certificat X509 de manera que comprova que el certificat sigui vlid, la persona s qui diu ser, no ha estat revocat. En aquest cas comprovem que es detecta un cas de certificat revocat i per tant invalid.
		Parmetres entrada: En aquesta prova passem com a parmetre dentrada el certificat personal revocat didentificaci: signaturadigital/cert/RevocatCARLOSVILAMATEOS.crt
		Parmetres dentrada: En aquesta prova passem com a parmetre dentrada la signatura del document en format CMS detached i tamb passem el propi document original.
		-	Signatura CMS detached: "signaturadigital/CPISR-1/ cvilama_platreball_portada (Revocat).p7b"
		-	Document original alterat: "signaturadigital/CPISR-1/ cvilama_platreball_portada (cms.doc Actiu).dat"
		Resultat: El resultat es guardar als mateixos objectes que els definits en la resta de test al objecte DigitalSignatureContentVo.		
		Per verificar que el test es satisfactori, ho comprovarem a travs dassertions, en aquest cas comprovarem que el missatge de detall del resultat de la validaci en el format DSS sigui igual a lexpressi que indica que el certificat est revocat:  				vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
		      assertTrue(minor.contains("invalid:certificate:Revoked")); 
     * @throws Exception
     */
    public void testX509CertificateRevoked() throws Exception
    {
        try
        {
            Validacio val = new ValidacioCertificat();
            DigitalSignatureContentVo vo = ((ValidacioCertificat)val).createDefaultVO();
            String certFile = propFile.getProperty(ConfigProperties.PROP_CERT_FILE_REVOKED);
            log.info("testing certFile=["+certFile+"]");               
            vo.setCertificate(Utils.getBytesFromFile(certFile));                            
            val.validar(vo); // cridar a validar classe pare i per lherncia i polimorfisme acabar cridant als mtodes especfics de ValidacioSignatura 
            String minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
            assertTrue(minor.contains("invalid:certificate:Revoked"));                        
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(); 
        }
    }
  
    /**
     * Prova amb el document pdf original alterat amb signatura detached 
     *  Missatge per a la validaci d'una signatura CMS detached, aqu tenim un document i provem un SignatureObject que cont una signatura binaria codificada en Base64. El document adjuntat en aquest cas pot ser el document en si codificat en base64 o be el seu resum criptogrfic tal i com disposa DSS tal fet.
		Aquest test valida una signatura CMS detached a partir del document original per alterat desprs del moment de la signatura. Com a document signat de base utilitzarem la portada daquesta memria cvilama_platreball_portada.pdf, el servei validar si el document realment est signat per qui diu ser, si es integra i per tant no ha estat modificat des de el moment de la signatura, si s aix la persona que ha signat no ho podr repudiar i podem assegurar que s qui ha signat el document i que aquest no ha estat modificat des daleshores. En aquest cas hem alterat el document signat i per tant el test correcte s detectar aquesta alteraci.     *
		Parmetres dentrada: En aquesta prova passem com a parmetre dentrada la signatura del document en format CMS detached i tamb passem el propi document original per alterat de forma que no s el mateix document que es va signar.
		Parmetres dentrada: En aquesta prova passem com a parmetre dentrada la signatura del document en format CMS detached i tamb passem el propi document original per alterat de forma que no s el mateix document que es va signar.
		-	Signatura CMS detached: "signaturadigital/CPISR-1/ cvilama_platreball_portada(cms.signature Actiu).dat"
		-	Document original alterat: "signaturadigital/CPISR-1/ cvilama_platreball_portada(Alterat).pdf"		
		Resultat: El resultat es guardar als mateixos objectes que els definits en la resta de test al objecte DigitalSignatureContentVo.
		Per verificar que el test es satisfactori, ho comprovarem a travs dassertions, en aquest cas comprovarem que el missatge de detall del resultat de la validaci en el format DSS sigui igual a lexpressi que indica que la signatura no s correcte:  				vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
		      assertTrue(minor.contains("invalid:incorrectSignature"));  
      
     * @throws Exception
     */
    public void testDetachedAltered() throws Exception
    {
        try
        {
            Validacio val = new ValidacioSignatura(TipusSignatura.CMSdetachedDoc);
            DigitalSignatureContentVo vo = ((ValidacioSignatura)val).createDefaultVO();
            vo.setSignature(Utils.readBase64File("signaturadigital/CPISR-1/cvilama_platreball_portada (cms.signature Actiu).dat"));
            vo.setData(Utils.getBytesFromFile("signaturadigital/CPISR-1/cvilama_platreball_portada (Alterat).pdf"));
            val.validar(vo); // cridar a validar classe pare i per lherncia i polimorfisme acabar cridant als mtodes especfics de ValidacioSignatura 
            String minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
            assertTrue(minor.contains("incorrectSignature"));                       
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(); 
        }
    }

    /**
     * Prova amb el document pdf original amb signatura detached sense alterar ambdos en codificacio BASE64 
     * Missatge per a la validaci d'una signatura CMS detached, aqu tenim un document i provem un SignatureObject que cont una signatura binaria codificada en Base64. El document adjuntat en aquest cas pot ser el document en si codificat en base64 o be el seu resum criptogrfic tal i com disposa DSS tal fet.
     * Aquest test valida un una signatura CMS detached a partir del document original codificat en Base64. Com a document signat de base utilitzarem la portada daquesta memria cvilama_platreball_portada.pdf, el servei validar si el document realment est signat per qui diu ser, si es integra i per tant no ha estat modificat des de el moment de la signatura, si s aix la persona que ha signat no ho podr repudiar i podem assegurar que s qui ha signat el document i que aquest no ha estat modificat des daleshores. 
	 *	Parmetres dentrada: En aquesta prova passem com a parmetre dentrada la signatura del document en format CMS detached i tamb passem el propi document original per alterat de forma que no s el mateix document que es va signar.
	 * 	Parmetres dentrada: En aquesta prova passem com a parmetre dentrada la signatura del document en format CMS detached i tamb passem el propi document original per alterat de forma que no s el mateix document que es va signar.
		-	Signatura CMS detached: "signaturadigital/CPISR-1/ cvilama_platreball_portada(cms.signature Actiu).dat"
		- 	Document original alterat: "signaturadigital/CPISR-1/ cvilama_platreball_portada(cms.docActiu).dat""
		Resultat: El resultat es guardar als mateixos objectes que els definits en la resta de test al objecte DigitalSignatureContentVo.		
		Per verificar que el test es satisfactori, ho comprovarem a travs dassertions, en aquest cas comprovarem que el missatge de detall del resultat de la validaci en el format DSS sigui igual a lexpressi que indica que la signatura s correcte:  				vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
		      assertTrue(minor.contains("valid:signature:onAllDocuments"));  

     * @throws Exception
     */
    public void testDetachedValid() throws Exception
    {
        try
        {
            Validacio val = new ValidacioSignatura(TipusSignatura.CMSdetachedDoc);
            DigitalSignatureContentVo vo = ((ValidacioSignatura)val).createDefaultVO();
            vo.setSignature(Utils.readBase64File("signaturadigital/CPISR-1/cvilama_platreball_portada (cms.signature Actiu).dat"));
            vo.setData(Utils.readBase64File("signaturadigital/CPISR-1/cvilama_platreball_portada (cms.doc Actiu).dat"));
            val.validar(vo);
            String minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
            assertTrue(minor.contains("valid:signature:onAllDocuments"));
                        
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(); 
        }
    }
        
    
    /**
     * Prova amb el document pdf original amb signatura Detached i Revocada 
     * Missatge per a la validaci d'una signatura CMS detached, aqu tenim un document i provem un SignatureObject que cont una signatura binaria codificada en Base64. El document adjuntat en aquest cas pot ser el document en si codificat en base64 o be el seu resum criptogrfic tal i com disposa DSS tal fet.
     * Aquest test valida una signatura CMS detached a partir del document original. Com a document signat de base utilitzarem la portada daquesta memria cvilama_platreball_portada.pdf, el servei validar si el document realment est signat per qui diu ser, si es integra i per tant no ha estat modificat des de el moment de la signatura, si s aix la persona que ha signat no ho podr repudiar i podem assegurar que s qui ha signat el document i que aquest no ha estat modificat des daleshores. En aquest cas hem signat amb un certificat que est revocat i per tant el test correcte s detectar aquesta revocaci, perqu en el fons la signatura s correcte i el document est correctament signat i no ha sigut alterat des daleshores.
     * Parmetres dentrada: En aquesta prova passem com a parmetre dentrada la signatura del document en format CMS detached i tamb passem el propi document original.
     * Resultat: El resultat es guardar als mateixos objectes que els definits en la resta de test al objecte DigitalSignatureContentVo.
		Per verificar que el test es satisfactori, ho comprovarem a travs dassertions, en aquest cas comprovarem que el missatge de detall del resultat de la validaci en el format DSS sigui igual a lexpressi que indica que la signatura no s correcte:  				vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
      assertTrue(minor.contains("invalid:untrustedKey"));                       

     * @throws Exception
     */
    public void testDetachedRevocat() throws Exception
    {
        try
        {
            Validacio val = new ValidacioSignatura(TipusSignatura.CMSdetachedDoc);
            DigitalSignatureContentVo vo = ((ValidacioSignatura)val).createDefaultVO();
            vo.setSignature(Utils.readBase64File("signaturadigital/CPISR-1/cvilama_platreball_portada (Revocat).p7b"));
            vo.setData(Utils.readBase64File("signaturadigital/CPISR-1/cvilama_platreball_portada (cms.doc Actiu).dat"));
            val.validar(vo);
            String minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
            assertTrue(minor.contains("invalid:untrustedKey"));
                        
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(); 
        }
    }
    
    /**
     * Prova de validaci del PDF amb signatura amb el certificat no confiable, sense reconeixer per una CA (Prueba 12.f)
     * Aquest test valida un una signatura CMS detached incrustada en un PDF i amb el servei especial de CATCert per validar fitxers PDF, utilitzant el perfil urn:oasis:names:tc:dss:1.0:profiles:DSS_PDF. Com a document signat de base utilitzarem la portada daquesta memria cvilama_platreball_portada.pdf, el servei validar si el document realment est signat per qui diu ser, si es integra i per tant no ha estat modificat des de el moment de la signatura, si s aix la persona que ha signat no ho podr repudiar i podem assegurar que s qui ha signat el document i que aquest no ha estat modificat des daleshores.
		En aquest cas em signat el document amb un certificat emes per nosaltres mateixos i que per tant no s de confiana ni reconegut per les CAs arrel.
		Parmetres dentrada:
		-	Document PDF signat amb la signatura incrustada: "signaturadigital/CPISR-1/ cvilama_platreball_portada_signada (NoConfianca).pdf"
		Resultat: El resultat es guardar als mateixos objectes que els definits en la resta de test al objecte DigitalSignatureContentVo.
		Per verificar que el test es satisfactori, ho comprovarem a travs dassertions, en aquest cas comprovarem que el missatge de detall del resultat de la validaci en el format DSS sigui igual a lexpressi que indica que la signatura no s de confiana:  				vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
      assertTrue(minor.contains("invalid:indeterminateKey"));
     *
     */
    public final void testPDFNoConfianza() 
    {
        try
        {
            Validacio val = new ValidacioSignaturaPDF();
            DigitalSignatureContentVo vo = new DigitalSignatureContentVo();
            vo.setData(Utils.getBytesFromFile("signaturadigital/CPISR-1/cvilama_platreball_portada_signada (NoConfianca).pdf"));
            vo.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS_PDF);
            val.validar(vo); // cridar a validar classe pare i per lherncia i polimorfisme acabar cridant als mtodes especfics de ValidacioSignatura
            String minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
            assertTrue(minor.contains("invalid:indeterminateKey"));
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(); 
        }
        
    }

    /**
     * Prova de validaci del PDF amb signatura amb el certificat actiu 
     * Aquest test valida un una signatura CMS detached incrustada en un PDF i amb el servei espcial de CATCert per validar fitxers PDF, utilitzant el perfil urn:oasis:names:tc:dss:1.0:profiles:DSS_PDF. Com a document signat de base utilitzarem la portada daquesta memria cvilama_platreball_portada.pdf, el servei validar si el document realment est signat per qui diu ser, si es integra i per tant no ha estat modificat des de el moment de la signatura, si s aix la persona que ha signat no ho podr repudiar i podem assegurar que s qui ha signat el document i que aquest no ha estat modificat des daleshores.
     * 	Parmetres dentrada:
		-	Document PDF signat amb la signatura incrustada: "signaturadigital/CPISR-1/ cvilama_platreball_portada_signada (Actiu).pdf"
     * Resultat: El resultat es guardar als mateixos objectes que els definits en la resta de test al objecte DigitalSignatureContentVo.
	 	Per verificar que el test es satisfactori, ho comprovarem a travs dassertions, en aquest cas comprovarem que el missatge de detall del resultat de la validaci en el format DSS sigui igual a lexpressi que indica que la signatura s correcte:  				vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
      assertTrue(minor.contains("valid:signature:onAllDocuments"));   
     *
     */
    public final void testPDFCPX() 
    {
        try
        {
            Validacio val = new ValidacioSignaturaPDF();
            DigitalSignatureContentVo vo = new DigitalSignatureContentVo();
            vo.setData(Utils.getBytesFromFile("signaturadigital/CPISR-1/cvilama_platreball_portada_signada (Actiu).pdf"));            
            vo.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS_PDF);
            val.validar(vo);
            String minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
            assertTrue(minor.contains("valid:signature:onAllDocuments"));
                        
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(); 
        }
        
    }
    
    /**
     * Prova de validaci del PDF amb signatura amb el certificat revocat
     * Aquest test valida un una signatura CMS detached incrustada en un PDF i amb el servei especial de CATCert per validar fitxers PDF, utilitzant el perfil urn:oasis:names:tc:dss:1.0:profiles:DSS_PDF. Com a document signat de base utilitzarem la portada daquesta memria cvilama_platreball_portada.pdf, el servei validar si el document realment est signat per qui diu ser, si es integra i per tant no ha estat modificat des de el moment de la signatura, si s aix la persona que ha signat no ho podr repudiar i podem assegurar que s qui ha signat el document i que aquest no ha estat modificat des daleshores. 
		En aquest cas em signat el document amb un certificat revocat.
		Parmetres dentrada:
		-	Document PDF signat amb la signatura incrustada: "signaturadigital/CPISR-1/ cvilama_platreball_portada (Revocat).pdf"
		Resultat: El resultat es guardar als mateixos objectes que els definits en la resta de test al objecte DigitalSignatureContentVo.
		Per verificar que el test es satisfactori, ho comprovarem a travs dassertions, en aquest cas comprovarem que el missatge de detall del resultat de la validaci en el format DSS sigui igual a lexpressi que indica que la signatura no s de confiana:  				vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);
      assertTrue(minor.contains("invalid:untrustedKey"));     
     *
     */    
    public final void testPDFRevocat() 
    {
        try
        {
            Validacio val = new ValidacioSignaturaPDF();
            DigitalSignatureContentVo vo = new DigitalSignatureContentVo();
            vo.setData(Utils.getBytesFromFile("signaturadigital/CPISR-1/cvilama_platreball_portada (Revocat).pdf"));
            vo.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS_PDF);
            val.validar(vo);
            String minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMinor);          
            assertTrue(minor.contains("invalid:untrustedKey"));
                        
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(); 
        }
        
    }
    
    public final void testTimestampPDF() 
    {
        try
        {
            CreacioSegellDeTemps cre = new CreacioSegellDeTemps();
            DigitalSignatureContentVo vo = new DigitalSignatureContentVo();
            vo.setCertificate(Utils.readBase64File("signaturadigital/timestamp/timestamp-certificate.dat"));
            vo.setPerfil(ConstantsApplicacio.PROTOCOL_perfilDSS_Timestmaping);

            //Obteniendo el hash del PDF
            byte[] buf = Utils.getBytesFromFile("signaturadigital/CPISR-1/cvilama_platreball_portada_signat (Segell).pdf");
            MessageDigest sha = MessageDigest.getInstance("SHA-1"); sha.update(buf);
            vo.setData(sha.digest());
            cre.crear(vo);           
            String minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMajor);
            assertTrue(minor.contains("Success"));
            ValidacioSegellDeTemps vals = new ValidacioSegellDeTemps();
            DigitalSignatureContentVo vo2=new DigitalSignatureContentVo();

            //Establecer el digest del documento que he de validar que coincide con el contenido en el sello de tiempo.
            sha=MessageDigest.getInstance("SHA-1");
            sha.update(Utils.getBytesFromFile("signaturadigital/CPISR-1/cvilama_platreball_portada_signat (Segell).pdf"));            
            vo2.setData(sha.digest());
            
            //Establecer el SignatureObject recibido por PSIS que se corresponde con ese sello de tiempo.
            vo2.setInlineXML((String)vo.getAtributs().get("dss:SignatureObject"));
            
            vals.validar(vo2); // cridar a validar classe pare i per lherncia i polimorfisme acabar cridant als mtodes especfics de ValidacioSignatura
            minor = (String) vo.getMsg().get(ConstantsApplicacio.MSG_ResultMajor);
            assertTrue(minor.contains("Success"));
        }
        catch (Exception e)
        {
            e.printStackTrace();
            fail(); 
        }
        
    }
}
