package com.steria.digitalsignature.config;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

public final class CommonUtils implements Serializable
{

    /**
     * 
     */
    private static final long serialVersionUID = 6366255861190004172L;
    
    /**
     * Mtode que passat un String i una llista de String et diu si el primer es troba en la llista
     * @param valor parmetre que es vol coneixer si hi &eacute;s o no en la llista
     * @param llistaValors llista de valors de tipus String
     * @return true si el valor existeix en la llista, false altrament o si algn dels 2 par&agrave;metres s'ha passat com a null
     */
    public static boolean isPresentInList(String valor, String[] llistaValors)
    {
        if (null == valor || null == llistaValors || 0 == llistaValors.length)
        {
            return false;
        }
        else
        {
            boolean trobat = false;
            for (String comparar: llistaValors)
            {
                trobat = comparar.equals(valor);
                if (trobat)
                {
                    break;
                }
            }
            return trobat;
        }
    }
    
    /**
     * Mtode que transforma un nmero a un String amb els 0 necessaris fins arribar al numero de caracters definits.
     * Si el numero es superior a aquest nombre de caracters, et retorna directament el numero transformat a String
     * @param numero El nmero al qual li vols possar zeros davant
     * @param nombreCaracters El nombre mxim de carcters que ha de tindre el String
     * @return un String amb nombreCaracters carcters possant zeros a l'esquerra del nmero passat com a parmetre.
     *         Si el nmero t ms caracters que el nombre mxim de caracters, llavors retorna el nmero passat a String
     */
    public static String convertToString(Number numero, int nombreCaracters)
    {
        if (null == numero )
        {
            return null;
        }
        String parcial = ""+numero;
        if (parcial.length() >= nombreCaracters)
        {
            return numero.toString();
        }
        StringBuffer result = new StringBuffer(nombreCaracters);
        for (int i = 0; i < nombreCaracters-parcial.length();i++)
        {
            result.append("0");
        }
        result.append(parcial);
        return result.toString();
    }

    /** Metodo para obtener todo el mensaje, incluidas las causas, de una exception */
    public static String getExceptionMessage(Throwable e)
    {
        String lsResp = "";
        if(e != null)
        {
            String cause=getExceptionMessage(e.getCause());
            if (!"".equals(cause)) cause="\n -- "+cause;
            lsResp = e.getClass().getName()+":"+ e.getMessage()+" " +cause ;
        }
        return lsResp;
    }

    /** Retorna el fichero de Properties con la configuracion de Contexto */
    public static Properties getContextConfig(String propertiesFile) throws IOException
    {
        InputStream file = null;
        Properties p = new Properties();

        URL url = Thread.currentThread().getContextClassLoader().getResource(propertiesFile);
        if (url != null)
        {
            file = url.openStream();
            p.load(file);
        }

        return p;
    }

    /**
     * 
     * @param srcs
     * @return
     */
    public static  <T> List<T>cleanNulls(List<T> srcs) {
        // Nos quedamos con los valores no nulos.
        List<T> nonNulls = new ArrayList<T>();
        for (T obj : srcs)
        {
            if (obj != null) {
                nonNulls.add(obj);
            }
        }
        return nonNulls;
    }
    
}
