package com.steria.digitalsignature.vo;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;

import com.steria.digitalsignature.config.ConstantsApplicacio;
import com.steria.digitalsignature.exception.SignaturaInvalidAttributExcepcio;
import com.steria.digitalsignature.messages.Messages;
/* 
 * @author: cvilamat
 * /**
 * @deprecated ja no s'utilitza pq ara noms s'utilitza un vo, DigitalSignatureContentVo, per a tots els casos de emmagatzematge de valors dels serveis PKI
 */

 @Deprecated public class X509CertificatVo extends DigitalSignatureContentVo implements Serializable, ConstantsApplicacio
{

    private static final long serialVersionUID = 4234773069424659290L;

    protected byte[] X509Certificate;

    /**
     Crear un objecte de validaci de certificat amb la solicitud de parmetres per defecte i el certificat configurat al fitxer de propietats com defecte.
     **/
    public X509CertificatVo()
    {
        initCertificatVo();
    }
    
    /**
     * Crear un objecte de validaci de certificat amb la solicitud de parmetres per defecte.
     * 
     * @param stream -- El contingut del Certificat de lusuari (X509Certificate.getEncoded())  o b en el cas duna sollicitud de segell de temps contindr el contingut del Certificat de la TSA que volem que ens emeti el segell (el segell tornar signat per la seva clau privada).
     * @param type - indica el tipus de dades que s'envien al stream de bytes (certificat, signatura, document, etc.)
     */
    public X509CertificatVo(byte[] stream)
    {
        initCertificatVo();
        this.X509Certificate = stream;
    }

    /**
     * @param stream
     * @param attributsOpcionals - un array amb els atributs opcionals a normalitzar i obtenir que es guardaran com a constants a la classes del component de constants daplicaci ApplicationConfig
     * @param operacionsAddicionals - un array amb altres atributs de configuraci com ara el perfil DSS
     * @param perfil - 
     * @throws SignaturaInvalidAttributExcepcio
     * @throws NullPointerException
     */
    public X509CertificatVo(byte[] stream, List<String> attributsOpcionals, List<String> operacionsAddicionals, String perfil) throws SignaturaInvalidAttributExcepcio, NullPointerException
    {
        // CAS X509Certificat - El contingut del Certificat de lusuari en base64 o b en el cas duna sollicitud de segell de temps contindr el contingut del Certificat de la TSA que volem que ens emeti el segell. 
        initCertificatVo();
        this.X509Certificate = stream;
        // afegim els atributs opcionals que ens pasen per parametre sempre i quant correspongin als definits - X509Certificat
        this.setAttributsOpcionals(attributsOpcionals);
        // afegim perfil (opcional) per un X509Certificat
        this.setPerfil(perfil);
        // afegim operacions opcionals que ens pasen per parametre sempre i quant correspongin als definits - X509Certificat
        this.setOperacionsAddicionals(operacionsAddicionals);
    }
    
    
    /**
     * Inicialitza la configuraci per defecte: atributs, operacions, etc.
     */
    public void initCertificatVo()
    {
        this.type = ConstantsApplicacio.TYPE.TYPE_X509Certificate;
        // posem el perfil ests (XSS)
        try { this.setPerfil(PROTOCOL_perfilDSS); } catch(SignaturaInvalidAttributExcepcio e) {}
        // afegim els atributs per defecte per un X509Certificat  - convertir arrays en lista
        this.attributsOpcionals.addAll(Arrays.asList(X509CertificateAtributsDefecte));
        // afegim operacions per defecte per un X509Certificat
        this.operacionsAddicionals.addAll(Arrays.asList(X509CertificateOperacionsDefecte));
    }
    
    /** afegeix a  la llista d'atributs opcionals a consultar a PSIS els elements indicats. 
     * La llista d'atributs consultables es al manual d'intregradors de PSIS */ 
    public void setAttributsOpcionals(List<String> attributsOpcionals) throws SignaturaInvalidAttributExcepcio
    {
        if (attributsOpcionals != null){
            // afegim els atributs opcionals que ens pasen per parametre sempre i quant correspongin als definits - X509Certificat
            this.attributsOpcionals.addAll(attributsOpcionals);
            
//            for (String attributOpcional : attributsOpcionals){
//                List<String> listX509CertificateAtributs = Arrays.asList(X509CertificateAtributs);
//                 if(listX509CertificateAtributs.contains(attributOpcional)) this.attributsOpcionals.add(attributOpcional);
//                else throw new SignaturaInvalidAttributExcepcio(Messages.getString("X509CertificatVo.0"));    
//            }
        }
    }

    /** Permet alterar la llista per defecte d'operacions PSIS sempre i quant estigui dintre de les posibles operacions contemplades */
    public void setOperacionsAddicionals(List<String> operacionsAddicionals) throws SignaturaInvalidAttributExcepcio
    {
        if (operacionsAddicionals != null){
            // afegim els atributs opcionals que ens pasen per parametre sempre i quant correspongin als definits - X509Certificat
            List<String> listX509CertificateOperacions = Arrays.asList(X509CertificateOperacions);
            for (String operacioOpcional : operacionsAddicionals){
                if(listX509CertificateOperacions.contains(operacioOpcional)) this.operacionsAddicionals.add(operacioOpcional);
                else throw new SignaturaInvalidAttributExcepcio(Messages.getString("X509CertificatVo.1"));    
            }
        }
    }

    /** Permet alterar el perfil PSIS per defecte sempre i quant estigui dintre dels posibles perfils contemplats */
     public void setPerfil(String perfil) throws SignaturaInvalidAttributExcepcio
    {
        // afegim perfil (opcional) per un X509Certificat
        List<String> listX509CertificateAtributs = Arrays.asList(X509CertificateAtributs);
        if(listX509CertificateAtributs.contains(perfil)) this.perfil=perfil;
        else throw new SignaturaInvalidAttributExcepcio(Messages.getString("X509CertificatVo.2")); 
    }
     
     /** obtenir el Stream que es objecte de validacio  (equivalent a  X509Certificate.getEncoded()) */
    public byte[] getX509Certificate()
    {
        return X509Certificate;
    }
    
    /**
     * Establir-el contingut del Certificat de l'usuari (X509Certificate.getEncoded())  o b en el cas d'una sollicitud de segell de temps contindr el contingut del Certificat de la TSA que volem que ens emeti el segell (el segell tornar signat per la seva clau privada).
     */
    public void setX509Certificate(byte[] certificate)
    {
        X509Certificate = certificate;
    } 
}
